import { WalkNode } from "./node";
export declare type Context<T> = {
    config: Config<T>;
    nodes: any;
    seenObjects: any[];
    callbacksByPosition: {
        [key: string]: BaseCallback[];
    };
};
export declare type NodeType = 'array' | 'object' | 'value';
export declare type DataStructureType = 'finiteTree' | 'tree' | 'graph' | 'infinite';
export declare type PositionType = 'preWalk' | 'postWalk';
export interface IOrderable {
    readonly executionOrder?: number;
}
export declare type Cb = (node: WalkNode) => void;
export declare type AsyncCb = Cb | ((node: WalkNode) => Promise<void>);
export declare type BaseCallback = IOrderable & {
    readonly executionOrder?: number;
    readonly positionFilters?: PositionType[];
    readonly keyFilters?: string[];
    readonly nodeTypeFilters?: NodeType[];
};
export declare type Callback = BaseCallback & {
    readonly callback: Cb;
};
export declare type AsyncCallback = BaseCallback & {
    readonly callback: AsyncCb;
};
export declare type Config<T extends BaseCallback> = {
    readonly traversalMode: 'depth' | 'breadth';
    readonly callbacks: T[];
    readonly graphMode: DataStructureType;
    readonly rootObjectCallbacks: boolean;
    readonly runCallbacks: boolean;
    readonly parallelizeAsyncCallbacks: boolean;
};
export declare type PartialConfig<T extends BaseCallback> = {
    readonly traversalMode?: 'depth' | 'breadth';
    readonly callbacks?: T[];
    readonly graphMode?: DataStructureType;
    readonly rootObjectCallbacks?: boolean;
    readonly runCallbacks?: boolean;
};
